@@ -1,12 +1,10 @@
+from __future__ import absolute_import
+
 import cython
-from Cython import __version__
+from .. import __version__
 
 import re, os, sys, time
-try:
-    from glob import iglob
-except ImportError:
-    # Py2.4
-    from glob import glob as iglob
+from glob import iglob
 
 try:
     import gzip
@@ -46,9 +44,9 @@ except ImportError:
 
 from distutils.extension import Extension
 
-from Cython import Utils
-from Cython.Utils import cached_function, cached_method, path_exists, find_root_package_dir
-from Cython.Compiler.Main import Context, CompilationOptions, default_options
+from .. import Utils
+from ..Utils import cached_function, cached_method, path_exists, find_root_package_dir
+from ..Compiler.Main import Context, CompilationOptions, default_options
 
 join_path = cached_function(os.path.join)
 
@@ -64,6 +62,7 @@ if sys.version_info[0] < 3:
 else:
     def encode_filename_in_py2(filename):
         return filename
+    basestring = str
 
 def extended_iglob(pattern):
     if '**/' in pattern:
@@ -148,14 +147,18 @@ distutils_settings = {
 
 @cython.locals(start=long, end=long)
 def line_iter(source):
-    start = 0
-    while True:
-        end = source.find('\n', start)
-        if end == -1:
-            yield source[start:]
-            return
-        yield source[start:end]
-        start = end+1
+    if isinstance(source, basestring):
+        start = 0
+        while True:
+            end = source.find('\n', start)
+            if end == -1:
+                yield source[start:]
+                return
+            yield source[start:end]
+            start = end+1
+    else:
+        for line in source:
+            yield line
 
 class DistutilsInfo(object):
 
@@ -225,6 +228,14 @@ class DistutilsInfo(object):
             resolved.values[key] = value
         return resolved
 
+    def apply(self, extension):
+        for key, value in self.values.items():
+            type = distutils_settings[key]
+            if type in [list, transitive_list]:
+                getattr(extension, key).extend(value)
+            else:
+                setattr(extension, key, value)
+
 @cython.locals(start=long, q=long, single_q=long, double_q=long, hash_mark=long,
                end=long, k=long, counter=long, quote_len=long)
 def strip_string_literals(code, prefix='__Pyx_L'):
@@ -573,6 +584,7 @@ def create_dependency_tree(ctx=None, quiet=False):
         _dep_tree = DependencyTree(ctx, quiet=quiet)
     return _dep_tree
 
+
 # This may be useful for advanced users?
 def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=False, exclude_failures=False):
     if not isinstance(patterns, (list, tuple)):
@@ -585,6 +597,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
         exclude = [exclude]
     for pattern in exclude:
         to_exclude.update(map(os.path.abspath, extended_iglob(pattern)))
+
     module_list = []
     for pattern in patterns:
         if isinstance(pattern, str):
@@ -594,8 +607,10 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
             base = None
             exn_type = Extension
         elif isinstance(pattern, Extension):
-            filepattern = pattern.sources[0]
-            if os.path.splitext(filepattern)[1] not in ('.py', '.pyx'):
+            for filepattern in pattern.sources:
+                if os.path.splitext(filepattern)[1] in ('.py', '.pyx'):
+                    break
+            else:
                 # ignore non-cython modules
                 module_list.append(pattern)
                 continue
@@ -605,6 +620,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
             exn_type = template.__class__
         else:
             raise TypeError(pattern)
+
         for file in extended_iglob(filepattern):
             if os.path.abspath(file) in to_exclude:
                 continue
@@ -615,6 +631,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                     continue
             else:
                 module_name = name
+
             if module_name not in seen:
                 try:
                     kwds = deps.distutils_info(file, aliases, base).values
@@ -626,9 +643,10 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                     for key, value in base.values.items():
                         if key not in kwds:
                             kwds[key] = value
+
                 sources = [file]
                 if template is not None:
-                    sources += template.sources[1:]
+                    sources += [m for m in template.sources if m != filepattern]
                 if 'sources' in kwds:
                     # allow users to add .c files etc.
                     for source in kwds['sources']:
@@ -642,6 +660,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                         # Always include everything from the template.
                         depends = list(set(template.depends).union(set(depends)))
                     kwds['depends'] = depends
+
                 module_list.append(exn_type(
                         name=module_name,
                         sources=sources,
@@ -650,6 +669,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                 seen.add(name)
     return module_list
 
+
 # This is the user-exposed entry point.
 def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, force=False,
               exclude_failures=False, **options):
@@ -694,6 +714,7 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
         aliases=aliases)
     deps = create_dependency_tree(ctx, quiet=quiet)
     build_dir = getattr(options, 'build_dir', None)
+
     modules_by_cfile = {}
     to_compile = []
     for m in module_list:
@@ -766,23 +787,43 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                 if build_dir:
                     copy_to_build_dir(source)
         m.sources = new_sources
+
     if hasattr(options, 'cache'):
         if not os.path.exists(options.cache):
             os.makedirs(options.cache)
     to_compile.sort()
+    if len(to_compile) <= 1:
+        nthreads = 0
     if nthreads:
         # Requires multiprocessing (or Python >= 2.6)
         try:
             import multiprocessing
-            pool = multiprocessing.Pool(nthreads)
+            pool = multiprocessing.Pool(
+                nthreads, initializer=_init_multiprocessing_helper)
         except (ImportError, OSError):
             print("multiprocessing required for parallel cythonization")
             nthreads = 0
         else:
-            pool.map(cythonize_one_helper, to_compile)
+            # This is a bit more involved than it should be, because KeyboardInterrupts
+            # break the multiprocessing workers when using a normal pool.map().
+            # See, for example:
+            # http://noswap.com/blog/python-multiprocessing-keyboardinterrupt
+            try:
+                result = pool.map_async(cythonize_one_helper, to_compile, chunksize=1)
+                pool.close()
+                while not result.ready():
+                    try:
+                        result.get(99999)  # seconds
+                    except multiprocessing.TimeoutError:
+                        pass
+            except KeyboardInterrupt:
+                pool.terminate()
+                raise
+            pool.join()
     if not nthreads:
         for args in to_compile:
             cythonize_one(*args[1:])
+
     if exclude_failures:
         failed_modules = set()
         for c_file, modules in modules_by_cfile.iteritems():
@@ -801,6 +842,7 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                 module_list.remove(module)
             print("Failed compilations: %s" % ', '.join(sorted([
                 module.name for module in failed_modules])))
+
     if hasattr(options, 'cache'):
         cleanup_cache(options.cache, getattr(options, 'cache_size', 1024 * 1024 * 100))
     # cythonize() is often followed by the (non-Python-buffered)
@@ -846,8 +888,8 @@ else:
 # TODO: Share context? Issue: pyx processing leaks into pxd module
 @record_results
 def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_failure=True):
-    from Cython.Compiler.Main import compile, default_options
-    from Cython.Compiler.Errors import CompileError, PyrexError
+    from ..Compiler.Main import compile, default_options
+    from ..Compiler.Errors import CompileError, PyrexError
 
     if fingerprint:
         if not os.path.exists(options.cache):
@@ -913,6 +955,7 @@ def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_f
         finally:
             f.close()
 
+
 def cythonize_one_helper(m):
     import traceback
     try:
@@ -921,6 +964,13 @@ def cythonize_one_helper(m):
         traceback.print_exc()
         raise
 
+
+def _init_multiprocessing_helper():
+    # KeyboardInterrupt kills workers, so don't let them get it
+    import signal
+    signal.signal(signal.SIGINT, signal.SIG_IGN)
+
+
 def cleanup_cache(cache, target_size, ratio=.85):
     try:
         p = subprocess.Popen(['du', '-s', '-k', os.path.abspath(cache)], stdout=subprocess.PIPE)
