@@ -2,23 +2,29 @@
 #   Cython Top Level
 #
 
-import os, sys, re, codecs
-if sys.version_info[:2] < (2, 4):
-    sys.stderr.write("Sorry, Cython requires Python 2.4 or later\n")
+from __future__ import absolute_import
+
+import os
+import re
+import sys
+import codecs
+
+if sys.version_info[:2] < (2, 6) or (3, 0) <= sys.version_info[:2] < (3, 2):
+    sys.stderr.write("Sorry, Cython requires Python 2.6+ or 3.2+, found %d.%d\n" % tuple(sys.version_info[:2]))
     sys.exit(1)
 
-import Errors
+from . import Errors
 # Do not import Parsing here, import it when needed, because Parsing imports
 # Nodes, which globally needs debug command line options initialized to set a
 # conditional metaclass. These options are processed by CmdLine called from
 # main() in this file.
 # import Parsing
-import Version
-from Scanning import PyrexScanner, FileSourceDescriptor
-from Errors import PyrexError, CompileError, error, warning
-from Symtab import ModuleScope
-from Cython import Utils
-import Options
+from .Scanning import PyrexScanner, FileSourceDescriptor
+from .Errors import PyrexError, CompileError, error, warning
+from .Symtab import ModuleScope
+from .. import __version__ as version
+from .. import Utils
+from . import Options
 
 module_name_pattern = re.compile(r"[A-Za-z_][A-Za-z0-9_]*(\.[A-Za-z_][A-Za-z0-9_]*)*$")
 
@@ -58,7 +64,7 @@ class Context(object):
         # an infinite loop.
         # Better code organization would fix it.
 
-        import Builtin, CythonScope
+        from . import Builtin, CythonScope
         self.modules = {"__builtin__" : Builtin.builtin_scope}
         self.cython_scope = CythonScope.create_cython_scope(self)
         self.modules["cython"] = self.cython_scope
@@ -81,14 +87,14 @@ class Context(object):
     def set_language_level(self, level):
         self.language_level = level
         if level >= 3:
-            from Future import print_function, unicode_literals, absolute_import
+            from .Future import print_function, unicode_literals, absolute_import
             self.future_directives.update([print_function, unicode_literals, absolute_import])
             self.modules['builtins'] = self.modules['__builtin__']
 
     # pipeline creation functions can now be found in Pipeline.py
 
     def process_pxd(self, source_desc, scope, module_name):
-        import Pipeline
+        from . import Pipeline
         if isinstance(source_desc, FileSourceDescriptor) and source_desc._file_type == 'pyx':
             source = CompilationSource(source_desc, module_name, os.getcwd())
             result_sink = create_default_resultobj(source, self.options)
@@ -123,7 +129,7 @@ class Context(object):
                 pos = (module_name, 0, 0)
             raise CompileError(pos,
                 "'%s' is not a valid module name" % module_name)
-        if "." not in module_name and relative_to:
+        if relative_to:
             if debug_find_module:
                 print("...trying relative import")
             scope = relative_to.lookup_submodule(module_name)
@@ -139,7 +145,7 @@ class Context(object):
             for name in module_name.split("."):
                 scope = scope.find_submodule(name)
         if debug_find_module:
-            print("...scope =", scope)
+            print("...scope = %s" % scope)
         if not scope.pxd_file_loaded:
             if debug_find_module:
                 print("...pxd not loaded")
@@ -149,20 +155,20 @@ class Context(object):
                     print("...looking for pxd file")
                 pxd_pathname = self.find_pxd_file(module_name, pos)
                 if debug_find_module:
-                    print("......found ", pxd_pathname)
+                    print("......found %s" % pxd_pathname)
                 if not pxd_pathname and need_pxd:
                     package_pathname = self.search_include_directories(module_name, ".py", pos)
                     if package_pathname and package_pathname.endswith('__init__.py'):
                         pass
                     else:
-                        error(pos, "'%s.pxd' not found" % module_name)
+                        error(pos, "'%s.pxd' not found" % module_name.replace('.', os.sep))
             if pxd_pathname:
                 try:
                     if debug_find_module:
                         print("Context.find_module: Parsing %s" % pxd_pathname)
                     rel_path = module_name.replace('.', os.sep) + os.path.splitext(pxd_pathname)[1]
                     if not pxd_pathname.endswith(rel_path):
-                        rel_path = pxd_pathname # safety measure to prevent printing incorrect paths
+                        rel_path = pxd_pathname  # safety measure to prevent printing incorrect paths
                     source_desc = FileSourceDescriptor(pxd_pathname, rel_path)
                     err, result = self.process_pxd(source_desc, scope, module_name)
                     if err:
@@ -294,7 +300,7 @@ class Context(object):
         try:
             f = Utils.open_source_file(source_filename, "rU")
             try:
-                import Parsing
+                from . import Parsing
                 s = PyrexScanner(f, source_desc, source_encoding = f.encoding,
                                  scope = scope, context = self)
                 tree = Parsing.p_module(s, pxd, full_module_name)
@@ -309,7 +315,14 @@ class Context(object):
             position = e.args[2]
             encoding = e.args[0]
 
-            for idx, c in enumerate(open(source_filename, "rb").read()):
+            f = open(source_filename, "rb")
+            try:
+                byte_data = f.read()
+            finally:
+                f.close()
+
+            # FIXME: make this at least a little less inefficient
+            for idx, c in enumerate(byte_data):
                 if c in (ord('\n'), '\n'):
                     line += 1
                     column = 0
@@ -384,7 +397,7 @@ def create_default_resultobj(compilation_source, options):
     return result
 
 def run_pipeline(source, options, full_module_name=None, context=None):
-    import Pipeline
+    from . import Pipeline
 
     source_ext = os.path.splitext(source)[1]
     options.configure_language_defaults(source_ext[1:]) # py/pyx
@@ -454,8 +467,8 @@ class CompilationOptions(object):
     include_path      [string]  Directories to search for include files
     output_file       string    Name of generated .c file
     generate_pxi      boolean   Generate .pxi file for public declarations
-    capi_reexport_cincludes  
-                      boolean   Add cincluded headers to any auto-generated 
+    capi_reexport_cincludes
+                      boolean   Add cincluded headers to any auto-generated
                                 header files.
     timestamps        boolean   Only compile changed source files.
     verbose           boolean   Always print source names being compiled
@@ -466,7 +479,7 @@ class CompilationOptions(object):
     cplus             boolean   Compile as c++ code
     """
 
-    def __init__(self, defaults = None, **kw):
+    def __init__(self, defaults=None, **kw):
         self.include_path = []
         if defaults:
             if isinstance(defaults, CompilationOptions):
@@ -477,6 +490,16 @@ class CompilationOptions(object):
         options = dict(defaults)
         options.update(kw)
 
+        # let's assume 'default_options' contains a value for most known compiler options
+        # and validate against them
+        unknown_options = set(options) - set(default_options)
+        # ignore valid options that are not in the defaults
+        unknown_options.difference_update(['include_path'])
+        if unknown_options:
+            raise ValueError("got unexpected compilation option%s: %s" % (
+                's' if len(unknown_options) > 1 else '',
+                ', '.join(unknown_options)))
+
         directives = dict(options['compiler_directives']) # copy mutable field
         options['compiler_directives'] = directives
         if 'language_level' in directives and 'language_level' not in kw:
@@ -612,14 +635,14 @@ def main(command_line = 0):
     args = sys.argv[1:]
     any_failures = 0
     if command_line:
-        from CmdLine import parse_command_line
+        from .CmdLine import parse_command_line
         options, sources = parse_command_line(args)
     else:
         options = CompilationOptions(default_options)
         sources = args
 
     if options.show_version:
-        sys.stderr.write("Cython version %s\n" % Version.version)
+        sys.stderr.write("Cython version %s\n" % version)
     if options.working_path!="":
         os.chdir(options.working_path)
     try:
@@ -662,4 +685,6 @@ default_options = dict(
     gdb_debug = False,
     compile_time_env = None,
     common_utility_include_dir = None,
+    output_dir=None,
+    build_dir=None,
 )
