@@ -1,11 +1,13 @@
-from Errors import error, message
-import ExprNodes
-import Nodes
-import Builtin
-import PyrexTypes
-from Cython import Utils
-from PyrexTypes import py_object_type, unspecified_type
-from Visitor import CythonTransform, EnvTransform
+from __future__ import absolute_import
+
+from .Errors import error, message
+from . import ExprNodes
+from . import Nodes
+from . import Builtin
+from . import PyrexTypes
+from .. import Utils
+from .PyrexTypes import py_object_type, unspecified_type
+from .Visitor import CythonTransform, EnvTransform
 
 
 class TypedExprNode(ExprNodes.ExprNode):
@@ -66,7 +68,7 @@ class MarkParallelAssignments(EnvTransform):
             pass
 
     def visit_WithTargetAssignmentStatNode(self, node):
-        self.mark_assignment(node.lhs, node.rhs)
+        self.mark_assignment(node.lhs, node.with_node.enter_call)
         self.visitchildren(node)
         return node
 
@@ -360,8 +362,8 @@ class SimpleAssignmentTypeInferer(object):
             return
 
         # Set of assignemnts
-        assignments = set([])
-        assmts_resolved = set([])
+        assignments = set()
+        assmts_resolved = set()
         dependencies = {}
         assmt_to_names = {}
 
@@ -383,6 +385,7 @@ class SimpleAssignmentTypeInferer(object):
             if not types:
                 node_type = py_object_type
             else:
+                entry = node.entry
                 node_type = spanning_type(
                     types, entry.might_overflow, entry.pos)
             node.inferred_type = node_type
@@ -392,6 +395,7 @@ class SimpleAssignmentTypeInferer(object):
                      if assmt.inferred_type is not None]
             if not types:
                 return
+            entry = node.entry
             return spanning_type(types, entry.might_overflow, entry.pos)
 
         def resolve_assignments(assignments):
@@ -404,10 +408,9 @@ class SimpleAssignmentTypeInferer(object):
                         infer_name_node_type(node)
                     # Resolve assmt
                     inferred_type = assmt.infer_type()
-                    done = False
                     assmts_resolved.add(assmt)
                     resolved.add(assmt)
-            assignments -= resolved
+            assignments.difference_update(resolved)
             return resolved
 
         def partial_infer(assmt):
@@ -427,13 +430,11 @@ class SimpleAssignmentTypeInferer(object):
             # try to handle circular references
             partials = set()
             for assmt in assignments:
-                partial_types = []
                 if assmt in partial_assmts:
                     continue
-                for node in assmt_to_names[assmt]:
-                    if partial_infer(assmt):
-                        partials.add(assmt)
-                        assmts_resolved.add(assmt)
+                if partial_infer(assmt):
+                    partials.add(assmt)
+                    assmts_resolved.add(assmt)
             partial_assmts.update(partials)
             return partials
 
@@ -450,7 +451,7 @@ class SimpleAssignmentTypeInferer(object):
             entry_type = py_object_type
             if assmts_resolved.issuperset(entry.cf_assignments):
                 types = [assmt.inferred_type for assmt in entry.cf_assignments]
-                if types and Utils.all(types):
+                if types and all(types):
                     entry_type = spanning_type(
                         types, entry.might_overflow, entry.pos)
                     inferred.add(entry)
@@ -542,7 +543,7 @@ def safe_spanning_type(types, might_overflow, pos):
         return result_type
     # TODO: double complex should be OK as well, but we need
     # to make sure everything is supported.
-    elif result_type.is_int and not might_overflow:
+    elif (result_type.is_int or result_type.is_enum) and not might_overflow:
         return result_type
     return py_object_type
 
