@@ -1,5 +1,5 @@
 # cython.* namespace for pure mode.
-__version__ = "0.20.1"
+__version__ = "0.21"
 
 
 # BEGIN shameless copy from Cython/minivect/minitypes.py
@@ -350,7 +350,7 @@ py_complex = typedef(complex, "double complex")
 
 # Predefined types
 
-int_types = ['char', 'short', 'Py_UNICODE', 'int', 'long', 'longlong', 'Py_ssize_t', 'size_t']
+int_types = ['char', 'short', 'Py_UNICODE', 'int', 'Py_UCS4', 'long', 'longlong', 'Py_ssize_t', 'size_t']
 float_types = ['longdouble', 'double', 'float']
 complex_types = ['longdoublecomplex', 'doublecomplex', 'floatcomplex', 'complex']
 other_types = ['bint', 'void']
@@ -368,7 +368,7 @@ gs = globals()
 for name in int_types:
     reprname = to_repr(name, name)
     gs[name] = typedef(py_int, reprname)
-    if name != 'Py_UNICODE' and not name.endswith('size_t'):
+    if name not in ('Py_UNICODE', 'Py_UCS4') and not name.endswith('size_t'):
         gs['u'+name] = typedef(py_int, "unsigned " + reprname)
         gs['s'+name] = typedef(py_int, "signed " + reprname)
 
