@@ -56,6 +56,8 @@ except ImportError:
             return self._dict[key]
         def __setitem__(self, key, value):
             self._dict[key] = value
+        def __contains__(self, key):
+            return key in self._dict
         def __repr__(self):
             return repr(self._dict)
         def __nonzero__(self):
@@ -75,6 +77,7 @@ from distutils.command.build_ext import build_ext as _build_ext
 from distutils import sysconfig
 distutils_distro = Distribution()
 
+
 if sys.platform == 'win32':
     # TODO: Figure out why this hackery (see http://thread.gmane.org/gmane.comp.python.cython.devel/8280/).
     config_files = distutils_distro.find_config_files()
@@ -92,6 +95,8 @@ EXT_DEP_MODULES = {
     'tag:pstats': 'pstats',
     'tag:posix' : 'posix',
     'tag:array' : 'array',
+    'tag:ipython': 'IPython',
+    'tag:jedi': 'jedi',
 }
 
 def patch_inspect_isfunction():
@@ -111,6 +116,77 @@ def unpatch_inspect_isfunction():
     else:
         inspect.isfunction = orig_isfunction
 
+def def_to_cdef(source):
+    '''
+    Converts the module-level def methods into cdef methods, i.e.
+
+        @decorator
+        def foo([args]):
+            """
+            [tests]
+            """
+            [body]
+
+    becomes
+
+        def foo([args]):
+            """
+            [tests]
+            """
+            return foo_c([args])
+
+        cdef foo_c([args]):
+            [body]
+    '''
+    output = []
+    skip = False
+    def_node = re.compile(r'def (\w+)\(([^()*]*)\):').match
+    lines = iter(source.split('\n'))
+    for line in lines:
+        if not line.strip():
+            output.append(line)
+            continue
+
+        if skip:
+            if line[0] != ' ':
+                skip = False
+            else:
+                continue
+
+        if line[0] == '@':
+            skip = True
+            continue
+
+        m = def_node(line)
+        if m:
+            name = m.group(1)
+            args = m.group(2)
+            if args:
+                args_no_types = ", ".join(arg.split()[-1] for arg in args.split(','))
+            else:
+                args_no_types = ""
+            output.append("def %s(%s):" % (name, args_no_types))
+            line = next(lines)
+            if '"""' in line:
+                has_docstring = True
+                output.append(line)
+                for line in lines:
+                    output.append(line)
+                    if '"""' in line:
+                        break
+            else:
+                has_docstring = False
+            output.append("    return %s_c(%s)" % (name, args_no_types))
+            output.append('')
+            output.append("cdef %s_c(%s):" % (name, args))
+            if not has_docstring:
+                output.append(line)
+
+        else:
+            output.append(line)
+
+    return '\n'.join(output)
+
 def update_linetrace_extension(ext):
     ext.define_macros.append(('CYTHON_TRACE', 1))
     return ext
@@ -204,7 +280,7 @@ EXCLUDE_EXT = object()
 EXT_EXTRAS = {
     'tag:numpy' : update_numpy_extension,
     'tag:openmp': update_openmp_extension,
-    'tag:trace':  update_linetrace_extension,
+    'tag:trace' : update_linetrace_extension,
 }
 
 
@@ -259,6 +335,7 @@ VER_DEP_MODULES = {
                                           'run.yield_inside_lambda',
                                           'run.test_dictviews',
                                           'run.pyclass_special_methods',
+                                          'run.set_literals',
                                           ]),
     # The next line should start (3,); but this is a dictionary, so
     # we can only have one (3,) key.  Since 2.7 is supposed to be the
@@ -280,14 +357,6 @@ VER_DEP_MODULES = {
                                           ]),
 }
 
-# files that should not be converted to Python 3 code with 2to3
-KEEP_2X_FILES = [
-    os.path.join('Cython', 'Debugger', 'Tests', 'test_libcython_in_gdb.py'),
-    os.path.join('Cython', 'Debugger', 'Tests', 'test_libpython_in_gdb.py'),
-    os.path.join('Cython', 'Debugger', 'libcython.py'),
-    os.path.join('Cython', 'Debugger', 'libpython.py'),
-]
-
 COMPILER = None
 INCLUDE_DIRS = [ d for d in os.getenv('INCLUDE', '').split(os.pathsep) if d ]
 CFLAGS = os.getenv('CFLAGS', '').split()
@@ -334,7 +403,7 @@ def parse_tags(filepath):
                 if tag == 'tags':
                     tag = 'tag'
                     print("WARNING: test tags use the 'tag' directive, not 'tags' (%s)" % filepath)
-                if tag not in ('mode', 'tag', 'ticket', 'cython'):
+                if tag not in ('mode', 'tag', 'ticket', 'cython', 'distutils', 'preparse'):
                     print("WARNING: unknown test directive '%s' found (%s)" % (tag, filepath))
                 values = values.split(',')
                 tags[tag].extend(filter(None, [value.strip() for value in values]))
@@ -418,7 +487,8 @@ class ErrorWriter(object):
 class TestBuilder(object):
     def __init__(self, rootdir, workdir, selectors, exclude_selectors, annotate,
                  cleanup_workdir, cleanup_sharedlibs, cleanup_failures,
-                 with_pyregr, cython_only, languages, test_bugs, fork, language_level):
+                 with_pyregr, cython_only, languages, test_bugs, fork, language_level,
+                 common_utility_dir):
         self.rootdir = rootdir
         self.workdir = workdir
         self.selectors = selectors
@@ -433,6 +503,7 @@ class TestBuilder(object):
         self.test_bugs = test_bugs
         self.fork = fork
         self.language_level = language_level
+        self.common_utility_dir = common_utility_dir
 
     def build_suite(self):
         suite = unittest.TestSuite()
@@ -478,7 +549,7 @@ class TestBuilder(object):
                      if match(fqmodule, tags) ]:
                 continue
             if self.exclude_selectors:
-                if [1 for match in self.exclude_selectors 
+                if [1 for match in self.exclude_selectors
                         if match(fqmodule, tags)]:
                     continue
 
@@ -512,7 +583,7 @@ class TestBuilder(object):
             if mode == 'run' and ext == '.py' and not self.cython_only:
                 # additionally test file in real Python
                 suite.addTest(PureDoctestTestCase(module, os.path.join(path, filename)))
-                
+
         return suite
 
     def build_tests(self, test_class, path, workdir, module, expect_errors, tags):
@@ -535,19 +606,25 @@ class TestBuilder(object):
         elif 'no-cpp' in tags['tag'] and 'cpp' in self.languages:
             languages = list(languages)
             languages.remove('cpp')
+
+        preparse_list = tags.get('preparse', ['id'])
         tests = [ self.build_test(test_class, path, workdir, module, tags,
-                                  language, expect_errors, warning_errors)
-                  for language in languages ]
+                                  language, expect_errors, warning_errors, preparse)
+                  for language in languages
+                  for preparse in preparse_list ]
         return tests
 
     def build_test(self, test_class, path, workdir, module, tags,
-                   language, expect_errors, warning_errors):
+                   language, expect_errors, warning_errors, preparse):
         language_workdir = os.path.join(workdir, language)
         if not os.path.exists(language_workdir):
             os.makedirs(language_workdir)
         workdir = os.path.join(language_workdir, module)
+        if preparse != 'id':
+            workdir += '_%s' % str(preparse)
         return test_class(path, workdir, module, tags,
                           language=language,
+                          preparse=preparse,
                           expect_errors=expect_errors,
                           annotate=self.annotate,
                           cleanup_workdir=self.cleanup_workdir,
@@ -556,18 +633,22 @@ class TestBuilder(object):
                           cython_only=self.cython_only,
                           fork=self.fork,
                           language_level=self.language_level,
-                          warning_errors=warning_errors)
+                          warning_errors=warning_errors,
+                          common_utility_dir=self.common_utility_dir)
 
 class CythonCompileTestCase(unittest.TestCase):
-    def __init__(self, test_directory, workdir, module, tags, language='c',
+    def __init__(self, test_directory, workdir, module, tags, language='c', preparse='id',
                  expect_errors=False, annotate=False, cleanup_workdir=True,
                  cleanup_sharedlibs=True, cleanup_failures=True, cython_only=False,
-                 fork=True, language_level=2, warning_errors=False):
+                 fork=True, language_level=2, warning_errors=False,
+                 common_utility_dir=None):
         self.test_directory = test_directory
         self.tags = tags
         self.workdir = workdir
         self.module = module
         self.language = language
+        self.preparse = preparse
+        self.name = module if self.preparse == "id" else "%s_%s" % (module, preparse)
         self.expect_errors = expect_errors
         self.annotate = annotate
         self.cleanup_workdir = cleanup_workdir
@@ -577,10 +658,11 @@ class CythonCompileTestCase(unittest.TestCase):
         self.fork = fork
         self.language_level = language_level
         self.warning_errors = warning_errors
+        self.common_utility_dir = common_utility_dir
         unittest.TestCase.__init__(self)
 
     def shortDescription(self):
-        return "compiling (%s) %s" % (self.language, self.module)
+        return "compiling (%s) %s" % (self.language, self.name)
 
     def setUp(self):
         from Cython.Compiler import Options
@@ -663,11 +745,16 @@ class CythonCompileTestCase(unittest.TestCase):
                 if is_related(filename)]
 
     def copy_files(self, test_directory, target_directory, file_list):
-        # use symlink on Unix, copy on Windows
-        try:
-            copy = os.symlink
-        except AttributeError:
-            copy = shutil.copy
+        if self.preparse and self.preparse != 'id':
+            preparse_func = globals()[self.preparse]
+            def copy(src, dest):
+                open(dest, 'w').write(preparse_func(open(src).read()))
+        else:
+            # use symlink on Unix, copy on Windows
+            try:
+                copy = os.symlink
+            except AttributeError:
+                copy = shutil.copy
 
         join = os.path.join
         for filename in file_list:
@@ -694,6 +781,7 @@ class CythonCompileTestCase(unittest.TestCase):
                     out.write(line)
         finally:
             source_and_output.close()
+
         try:
             geterrors = out.geterrors
         except AttributeError:
@@ -709,17 +797,28 @@ class CythonCompileTestCase(unittest.TestCase):
             include_dirs.append(incdir)
         source = self.find_module_source_file(
             os.path.join(test_directory, module + '.pyx'))
+        if self.preparse == 'id':
+            source = self.find_module_source_file(
+                os.path.join(test_directory, module + '.pyx'))
+        else:
+            self.copy_files(test_directory, targetdir, [module + '.pyx'])
+            source = os.path.join(targetdir, module + '.pyx')
         target = os.path.join(targetdir, self.build_target_filename(module))
 
         if extra_compile_options is None:
             extra_compile_options = {}
 
+        if 'allow_unknown_names' in self.tags['tag']:
+            from Cython.Compiler import Options
+            Options.error_on_unknown_names = False
+
         try:
             CompilationOptions
         except NameError:
             from Cython.Compiler.Main import CompilationOptions
             from Cython.Compiler.Main import compile as cython_compile
             from Cython.Compiler.Main import default_options
+        common_utility_include_dir = self.common_utility_dir
 
         options = CompilationOptions(
             default_options,
@@ -731,6 +830,7 @@ class CythonCompileTestCase(unittest.TestCase):
             language_level = self.language_level,
             generate_pxi = False,
             evaluate_tree_assertions = True,
+            common_utility_include_dir = common_utility_include_dir,
             **extra_compile_options
             )
         cython_compile(source, options=options,
@@ -772,6 +872,11 @@ class CythonCompileTestCase(unittest.TestCase):
                 # Set the language now as the fixer might need it
                 extension.language = 'c++'
 
+            if 'distutils' in self.tags:
+                from Cython.Build.Dependencies import DistutilsInfo
+                pyx_path = os.path.join(self.test_directory, self.module + ".pyx")
+                DistutilsInfo(open(pyx_path)).apply(extension)
+
             for matcher, fixer in list(EXT_EXTRAS.items()):
                 if isinstance(matcher, str):
                     # lazy init
@@ -827,7 +932,15 @@ class CythonCompileTestCase(unittest.TestCase):
             finally:
                 sys.stderr = old_stderr
 
-        if errors or expected_errors:
+        tostderr = sys.__stderr__.write
+        if 'cerror' in self.tags['tag']:
+            if errors:
+                tostderr("\n=== Expected C compile error ===\n")
+                tostderr("\n=== Got Cython errors: ===\n")
+                tostderr('\n'.join(errors))
+                tostderr('\n\n')
+                raise RuntimeError('should have generated extension code')
+        elif errors or expected_errors:
             try:
                 for expected, error in zip(expected_errors, errors):
                     self.assertEquals(expected, error)
@@ -838,20 +951,46 @@ class CythonCompileTestCase(unittest.TestCase):
                     unexpected_error = errors[len(expected_errors)]
                     self.assertEquals(None, unexpected_error)
             except AssertionError:
-                print("\n=== Expected errors: ===")
-                print('\n'.join(expected_errors))
-                print("\n\n=== Got errors: ===")
-                print('\n'.join(errors))
-                print('\n')
+                tostderr("\n=== Expected errors: ===\n")
+                tostderr('\n'.join(expected_errors))
+                tostderr("\n\n=== Got errors: ===\n")
+                tostderr('\n'.join(errors))
+                tostderr('\n\n')
                 raise
             return None
 
-        if self.cython_only:
-            so_path = None
-        else:
-            so_path = self.run_distutils(test_directory, module, workdir, incdir)
+        so_path = None
+        if not self.cython_only:
+            from Cython.Utils import captured_fd, print_bytes
+            show_output = True
+            get_stderr = get_stdout = None
+            try:
+                with captured_fd(1) as get_stdout:
+                    with captured_fd(2) as get_stderr:
+                        so_path = self.run_distutils(test_directory, module, workdir, incdir)
+            except Exception:
+                if 'cerror' in self.tags['tag'] and get_stderr and get_stderr():
+                    show_output = False  # expected C compiler failure
+                else:
+                    raise
+            else:
+                if 'cerror' in self.tags['tag']:
+                    raise RuntimeError('should have failed C compile')
+            finally:
+                if show_output:
+                    stdout = get_stdout and get_stdout().strip()
+                    if stdout:
+                        tostderr("\n=== C/C++ compiler output: ===\n")
+                        print_bytes(stdout, end=None, file=sys.__stderr__)
+                    stderr = get_stderr and get_stderr().strip()
+                    if stderr:
+                        tostderr("\n=== C/C++ compiler error output: ===\n")
+                        print_bytes(stderr, end=None, file=sys.__stderr__)
+                    if stdout or stderr:
+                        tostderr("\n==============================\n")
         return so_path
 
+
 class CythonRunTestCase(CythonCompileTestCase):
     def setUp(self):
         CythonCompileTestCase.setUp(self)
@@ -862,7 +1001,7 @@ class CythonRunTestCase(CythonCompileTestCase):
         if self.cython_only:
             return CythonCompileTestCase.shortDescription(self)
         else:
-            return "compiling (%s) and running %s" % (self.language, self.module)
+            return "compiling (%s) and running %s" % (self.language, self.name)
 
     def run(self, result=None):
         if result is None:
@@ -874,7 +1013,7 @@ class CythonRunTestCase(CythonCompileTestCase):
                 self.success = False
                 ext_so_path = self.runCompileTest()
                 failures, errors = len(result.failures), len(result.errors)
-                if not self.cython_only:
+                if not self.cython_only and ext_so_path is not None:
                     self.run_tests(result, ext_so_path)
                 if failures == len(result.failures) and errors == len(result.errors):
                     # No new errors...
@@ -1064,7 +1203,7 @@ class PartialTestResult(_TextTestResult):
 
 class CythonUnitTestCase(CythonRunTestCase):
     def shortDescription(self):
-        return "compiling (%s) tests in %s" % (self.language, self.module)
+        return "compiling (%s) tests in %s" % (self.language, self.name)
 
     def run_tests(self, result, ext_so_path):
         module = import_ext(self.module, ext_so_path)
@@ -1135,8 +1274,10 @@ class CythonPyregrTestCase(CythonRunTestCase):
 
         run_forked_test(result, run_test, self.shortDescription(), self.fork)
 
+
 include_debugger = IS_CPYTHON and sys.version_info[:2] > (2, 5)
 
+
 def collect_unittests(path, module_prefix, suite, selectors, exclude_selectors):
     def file_matches(filename):
         return filename.startswith("Test") and filename.endswith(".py")
@@ -1167,9 +1308,9 @@ def collect_unittests(path, module_prefix, suite, selectors, exclude_selectors):
                 if file_matches(f):
                     filepath = os.path.join(dirpath, f)[:-len(".py")]
                     modulename = module_prefix + filepath[len(path)+1:].replace(os.path.sep, '.')
-                    if not [ 1 for match in selectors if match(modulename) ]:
+                    if not any(1 for match in selectors if match(modulename)):
                         continue
-                    if [ 1 for match in exclude_selectors if match(modulename) ]:
+                    if any(1 for match in exclude_selectors if match(modulename)):
                         continue
                     module = __import__(modulename)
                     for x in modulename.split('.')[1:]:
@@ -1177,7 +1318,6 @@ def collect_unittests(path, module_prefix, suite, selectors, exclude_selectors):
                     suite.addTests([loader.loadTestsFromModule(module)])
 
 
-
 def collect_doctests(path, module_prefix, suite, selectors, exclude_selectors):
     def package_matches(dirname):
         if dirname == 'Debugger' and not include_debugger:
@@ -1396,7 +1536,7 @@ class TagsSelector:
     def __init__(self, tag, value):
         self.tag = tag
         self.value = value
-    
+
     def __call__(self, testname, tags=None):
         if tags is None:
             return False
@@ -1404,7 +1544,7 @@ class TagsSelector:
             return self.value in tags[self.tag]
 
 class RegExSelector:
-    
+
     def __init__(self, pattern_string):
         try:
             self.pattern = re.compile(pattern_string, re.I|re.U)
@@ -1432,7 +1572,7 @@ class ShardExcludeSelector:
 
     def __call__(self, testname, tags=None):
         return abs(hash(testname)) % self.shard_count != self.shard_num
-        
+
 
 def refactor_for_py3(distdir, cy3_dir):
     # need to convert Cython sources first
@@ -1454,15 +1594,14 @@ def refactor_for_py3(distdir, cy3_dir):
                      recursive-include Cython/Debugger/Tests *
                      recursive-include Cython/Utility *
                      recursive-exclude pyximport test
+                     include Tools/*.py
                      include pyximport/*.py
                      include runtests.py
                      include cython.py
+                     include cythonize.py
                      ''')
     sys.path.insert(0, cy3_dir)
 
-    for keep_2x_file in KEEP_2X_FILES:
-        destfile = os.path.join(cy3_dir, keep_2x_file)
-        shutil.copy(keep_2x_file, destfile)
 
 class PendingThreadsError(RuntimeError):
     pass
@@ -1633,11 +1772,12 @@ def main():
                       help="use pyximport to automatically compile imported .pyx and .py files")
     parser.add_option("--watermark", dest="watermark", default=None,
                       help="deterministic generated by string")
+    parser.add_option("--use_common_utility_dir", default=False, action="store_true")
 
     options, cmd_args = parser.parse_args()
 
     WORKDIR = os.path.abspath(options.work_dir)
-    
+
     if sys.version_info[0] >= 3:
         options.doctests = False
         if options.with_cython:
@@ -1736,7 +1876,7 @@ def runtests(options, cmd_args, coverage=None):
 
     if options.shard_num > -1:
         WORKDIR = os.path.join(WORKDIR, str(options.shard_num))
-    
+
     # RUN ALL TESTS!
     UNITTEST_MODULE = "Cython"
     UNITTEST_ROOT = os.path.join(os.path.dirname(__file__), UNITTEST_MODULE)
@@ -1795,13 +1935,25 @@ def runtests(options, cmd_args, coverage=None):
     if not selectors:
         selectors = [ lambda x, tags=None: True ]
 
-    # Chech which external modules are not present and exclude tests
+    # Check which external modules are not present and exclude tests
     # which depends on them (by prefix)
 
     missing_dep_excluder = MissingDependencyExcluder(EXT_DEP_MODULES)
     version_dep_excluder = VersionDependencyExcluder(VER_DEP_MODULES)
     exclude_selectors = [missing_dep_excluder, version_dep_excluder] # want to print msg at exit
 
+    try:
+        import IPython.core.release
+        if list(IPython.core.release._ver) < [1, 0, 0]:
+            raise ImportError
+    except (ImportError, AttributeError, TypeError):
+        exclude_selectors.append(RegExSelector('IPython'))
+
+    try:
+        import jedi
+    except ImportError:
+        exclude_selectors.append(RegExSelector('Jedi'))
+
     if options.exclude:
         exclude_selectors += [ string_selector(r) for r in options.exclude ]
 
@@ -1834,6 +1986,13 @@ def runtests(options, cmd_args, coverage=None):
         sys.stderr.write("Backends: %s\n" % ','.join(backends))
     languages = backends
 
+    if options.use_common_utility_dir:
+        common_utility_dir = os.path.join(WORKDIR, 'utility_code')
+        if not os.path.exists(common_utility_dir):
+            os.makedirs(common_utility_dir)
+    else:
+        common_utility_dir = None
+
     sys.stderr.write("\n")
 
     test_suite = unittest.TestSuite()
@@ -1850,7 +2009,8 @@ def runtests(options, cmd_args, coverage=None):
                                 options.cleanup_sharedlibs, options.cleanup_failures,
                                 options.pyregr,
                                 options.cython_only, languages, test_bugs,
-                                options.fork, options.language_level)
+                                options.fork, options.language_level,
+                                common_utility_dir)
         test_suite.addTest(filetests.build_suite())
 
     if options.system_pyregr and languages:
@@ -1861,7 +2021,8 @@ def runtests(options, cmd_args, coverage=None):
                                     options.cleanup_sharedlibs, options.cleanup_failures,
                                     True,
                                     options.cython_only, languages, test_bugs,
-                                    options.fork, sys.version_info[0])
+                                    options.fork, sys.version_info[0],
+                                    common_utility_dir)
             sys.stderr.write("Including CPython regression tests in %s\n" % sys_pyregr_dir)
             test_suite.addTest(filetests.handle_directory(sys_pyregr_dir, 'pyregr'))
 
@@ -1879,13 +2040,25 @@ def runtests(options, cmd_args, coverage=None):
 
     result = test_runner.run(test_suite)
 
+    if common_utility_dir and options.shard_num < 0 and options.cleanup_workdir:
+        shutil.rmtree(common_utility_dir)
+
     if coverage is not None:
         coverage.stop()
-        ignored_modules = ('Options', 'Version', 'DebugFlags', 'CmdLine')
-        modules = [ module for name, module in sys.modules.items()
-                    if module is not None and
-                    name.startswith('Cython.Compiler.') and
-                    name[len('Cython.Compiler.'):] not in ignored_modules ]
+        ignored_modules = set(
+            'Cython.Compiler.' + name
+            for name in ('Version', 'DebugFlags', 'CmdLine')) | set(
+            'Cython.' + name
+            for name in ('Debugging',))
+        ignored_packages = ['Cython.Runtime', 'Cython.Tempita']
+        modules = [
+            module for name, module in sys.modules.items()
+            if module is not None and
+            name.startswith('Cython.') and
+            '.Tests' not in name and
+            name not in ignored_modules and
+            not any(name.startswith(package) for package in ignored_packages)
+        ]
         if options.coverage:
             coverage.report(modules, show_missing=0)
         if options.coverage_xml:
